﻿
    class XamTileManagerHelper : DependencyObject
    {
        #region Context
        public static readonly DependencyProperty ContextProperty =
            DependencyProperty.RegisterAttached(
                "Context",
                typeof(IXamTileManagerDisplayContext),
                typeof(XamTileManagerHelper),
                new PropertyMetadata(new XamTileManagerDisplayContext(), ContextChanged));

        public static IXamTileManagerDisplayContext GetContext(DependencyObject obj)
        {
            return (IXamTileManagerDisplayContext)obj.GetValue(ContextProperty);
        }

        public static void SetContext(DependencyObject obj, IXamTileManagerDisplayContext value)
        {
            obj.SetValue(ContextProperty, value);
        }

        static CustomTileManager xamTileManager;

        static IXamTileManagerDisplayContext context;

        static void ContextChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            xamTileManager = dependencyObject as CustomTileManager;

            if (xamTileManager == null)
            {
                return;
            }

            context = GetContext(xamTileManager);

            if (e.OldValue != null)
            {
                Unsubscribe();
            }

            if (xamTileManager != null && context != null)
            {
                Subscribe();
                LoadContext();
            }
        }

        static void Subscribe()
        {
            xamTileManager.TileStateChanged += XamTileManagerItemsSizeChanged;
        }

        static void Unsubscribe()
        {
            xamTileManager.TileStateChanged -= XamTileManagerItemsSizeChanged;
        }

        static void XamTileManagerItemsSizeChanged(object sender, TileStateChangedEventArgs tileStateChangedEventArgs)
        {
            SaveContext();
        }

        static void SaveContext()
        {
            Unsubscribe();

            var needToSaveContext =
                xamTileManager.MaximizedItems.Select(item => xamTileManager.GetItemInfo(item))
                    .Any(info => !info.IsClosed);

            if (needToSaveContext)
            {
                var maximizedIndexs = new List<int>();

                // ReSharper disable once LoopCanBeConvertedToQuery
                foreach (var maximizedItem in xamTileManager.MaximizedItems)
                {
                    var currentTileContent = maximizedItem as IXamTileItemViewModel;
                    // ReSharper disable once PossibleNullReferenceException
                    maximizedIndexs.Add(currentTileContent.Id);
                }

                context.MaximizedIndexs = maximizedIndexs;

                var settings = new PersistenceSettings
                {
                    SavePersistenceOptions = Infragistics.Persistence.Primitives.PersistenceOption.AllButIgnored
                };

                var memoryStream = PersistenceManager.Save(xamTileManager, settings);
                var reader = new StreamReader(memoryStream);
                var outputText = reader.ReadToEnd();
                reader.Close();

                context.PersistenceSettings = outputText;
            }

            Subscribe();
        }

        static void LoadContext()
        {
            var currentDataContext = xamTileManager.DataContext as ResultGridViewModel;
            if (currentDataContext == null)
                return;
            if (context.PersistenceSettings != null)
            {
                var stream = new MemoryStream();
                var writer = new StreamWriter(stream);
                writer.Write(context.PersistenceSettings);
                writer.Flush();
                stream.Position = 0;
                PersistenceManager.Load(xamTileManager, stream);
                xamTileManager.MaximizedTileLimit = 2;
                xamTileManager.NormalModeSettings.TileConstraints = new TileConstraints
                {
                    PreferredHeight = 10,
                    PreferredWidth = 10,
                    MaxHeight = 670,
                    MaxWidth = 890
                };
                xamTileManager.MaximizedModeSettings.MinimizedTileConstraints = new TileConstraints()
                {
                    PreferredHeight = 43,
                    PreferredWidth = 170
                };
                foreach (var item in xamTileManager.Items)
                {
                    var tileToMaximize = xamTileManager.TileFromItem(item);
                    var currentTileContent = tileToMaximize.Content as ResultGridItemViewModel;
                    if (currentTileContent != null &&
                        currentDataContext.Context.MaximizedIndexs.Contains(currentTileContent.Id))
                    {
                        tileToMaximize.IsMaximized = true;
                    }
                }
                xamTileManager.UpdateLayout();
            }
            else
            {
                SetToDefaultLayout();
            }
        }

        static void SetToDefaultLayout()
        {
            var currentDataContext = xamTileManager.DataContext as ResultGridViewModel;
            if (currentDataContext == null || !currentDataContext.Items.IsNullOrEmpty())
            {
                return;
            }

            switch (currentDataContext.Items.Count)
            {
                case 4:
                    xamTileManager.NormalModeSettings.TileConstraints = new TileConstraints
                    {
                        PreferredHeight = 10,
                        PreferredWidth = 10,
                        MaxHeight = 670,
                        MaxWidth = 890
                    };
                    xamTileManager.MaximizedModeSettings.MinimizedTileConstraints = new TileConstraints()
                    {
                        PreferredHeight = 43,
                        PreferredWidth = 170
                    };
                    break;
                case 3:
                    xamTileManager.NormalModeSettings.TileConstraints = new TileConstraints
                    {
                        PreferredHeight = 100,
                        PreferredWidth = 100,
                        MaxHeight = 470,
                        MaxWidth = 690
                    };
                    break;
                case 1:
                    xamTileManager.NormalModeSettings.TileConstraints = new TileConstraints
                    {
                        MaxHeight = 1000,
                        MaxWidth = 1000
                    };
                    var curentTile = xamTileManager.TileFromItem(currentDataContext.Items[0]);
                    curentTile.MaximizeButtonVisibility = Visibility.Collapsed;
                    curentTile.CloseButtonVisibility = Visibility.Collapsed;
                    break;
            }
        }
        #endregion

        #region SelectedItem
        public static readonly DependencyProperty SelectedItemProperty =
            DependencyProperty.RegisterAttached(
                "SelectedItem",
                typeof(ResultGridItemViewModel),
                typeof(XamTileManagerHelper),
                new PropertyMetadata(null, SelectedItemChanged));

        static ResultGridItemViewModel selectedItem;

        public static ResultGridItemViewModel GetSelectedItem(DependencyObject obj)
        {
            return (ResultGridItemViewModel)obj.GetValue(SelectedItemProperty);
        }

        public static void SetSelectedItem(DependencyObject obj, ResultGridItemViewModel value)
        {
            obj.SetValue(SelectedItemProperty, value);
        }

        static void SelectedItemChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            xamTileManager = dependencyObject as CustomTileManager;

            if (xamTileManager == null)
            {
                return;
            }

            selectedItem = GetSelectedItem(xamTileManager);

            if (e.OldValue != null)
            {
                SaveSelectedItem();
            }

            if (xamTileManager != null && selectedItem != null)
            {
                LoadSelectedItem();
            }
        }

        static void SaveSelectedItem()
        {
            Unsubscribe();

            var currentDataContext = xamTileManager.DataContext as ResultGridViewModel;
            if (currentDataContext == null)
                return;
            
            currentDataContext.SelectedItem = GetSelectedItem(xamTileManager);

            Subscribe();
        }

        static void LoadSelectedItem()
        {
            var currentDataContext = xamTileManager.DataContext as ResultGridViewModel;
            if (currentDataContext == null)
                return;
            if (currentDataContext.SelectedItem != null)
            {
                SetSelectedItem(xamTileManager, currentDataContext.SelectedItem);
            }
            else
            {
                SetToDefaultLayout();
            }
        }

        #endregion
    }
}
